@@ -165,7 +165,6 @@ nla_put_failure:
 }
 
 static int handle_interface_add(struct nl80211_state *state,
-				struct nl_cb *cb,
 				struct nl_msg *msg,
 				int argc, char **argv,
 				enum id_input id)
@@ -256,7 +255,6 @@ COMMAND(interface, add, "<name> type <type> [mesh_id <meshid>] [4addr on|off] [f
 	NL80211_CMD_NEW_INTERFACE, 0, CIB_NETDEV, handle_interface_add, NULL);
 
 static int handle_interface_del(struct nl80211_state *state,
-				struct nl_cb *cb,
 				struct nl_msg *msg,
 				int argc, char **argv,
 				enum id_input id)
@@ -374,19 +372,17 @@ static int print_iface_handler(struct nl_msg *msg, void *arg)
 }
 
 static int handle_interface_info(struct nl80211_state *state,
-				 struct nl_cb *cb,
 				 struct nl_msg *msg,
 				 int argc, char **argv,
 				 enum id_input id)
 {
-	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_iface_handler, NULL);
+	register_handler(print_iface_handler, NULL);
 	return 0;
 }
 TOPLEVEL(info, NULL, NL80211_CMD_GET_INTERFACE, 0, CIB_NETDEV, handle_interface_info,
 	 "Show information for this interface.");
 
 static int handle_interface_set(struct nl80211_state *state,
-				struct nl_cb *cb,
 				struct nl_msg *msg,
 				int argc, char **argv,
 				enum id_input id)
@@ -417,7 +413,6 @@ COMMAND(set, monitor, "<flag>*",
 	VALID_FLAGS);
 
 static int handle_interface_meshid(struct nl80211_state *state,
-				   struct nl_cb *cb,
 				   struct nl_msg *msg,
 				   int argc, char **argv,
 				   enum id_input id)
@@ -441,20 +436,18 @@ COMMAND(set, meshid, "<meshid>",
 static unsigned int dev_dump_wiphy;
 
 static int handle_dev_dump(struct nl80211_state *state,
-			   struct nl_cb *cb,
 			   struct nl_msg *msg,
 			   int argc, char **argv,
 			   enum id_input id)
 {
 	dev_dump_wiphy = -1;
-	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_iface_handler, &dev_dump_wiphy);
+	register_handler(print_iface_handler, &dev_dump_wiphy);
 	return 0;
 }
 TOPLEVEL(dev, NULL, NL80211_CMD_GET_INTERFACE, NLM_F_DUMP, CIB_NONE, handle_dev_dump,
 	 "List all network interfaces for wireless hardware.");
 
 static int handle_interface_type(struct nl80211_state *state,
-				 struct nl_cb *cb,
 				 struct nl_msg *msg,
 				 int argc, char **argv,
 				 enum id_input id)
@@ -481,7 +474,6 @@ COMMAND(set, type, "<type>",
 	IFACE_TYPES);
 
 static int handle_interface_4addr(struct nl80211_state *state,
-				  struct nl_cb *cb,
 				  struct nl_msg *msg,
 				  int argc, char **argv,
 				  enum id_input id)
@@ -495,7 +487,6 @@ COMMAND(set, 4addr, "<on|off>",
 	"Set interface 4addr (WDS) mode.");
 
 static int handle_interface_noack_map(struct nl80211_state *state,
-				      struct nl_cb *cb,
 				      struct nl_msg *msg,
 				      int argc, char **argv,
 				      enum id_input id)
@@ -523,7 +514,6 @@ COMMAND(set, noack_map, "<map>",
 
 
 static int handle_interface_wds_peer(struct nl80211_state *state,
-				     struct nl_cb *cb,
 				     struct nl_msg *msg,
 				     int argc, char **argv,
 				     enum id_input id)
@@ -555,7 +545,6 @@ COMMAND(set, peer, "<MAC address>",
 	"Set interface WDS peer.");
 
 static int set_mcast_rate(struct nl80211_state *state,
-			  struct nl_cb *cb,
 			  struct nl_msg *msg,
 			  int argc, char **argv,
 			  enum id_input id)
