@@ -76,7 +76,6 @@ static const char *dfs_domain_name(enum nl80211_dfs_regions region)
 }
 
 static int handle_reg_set(struct nl80211_state *state,
-			  struct nl_cb *cb,
 			  struct nl_msg *msg,
 			  int argc, char **argv,
 			  enum id_input id)
@@ -206,7 +205,7 @@ static int print_reg_handler(struct nl_msg *msg, void *arg)
 		PARSE_FLAG(NL80211_RRF_DFS, "DFS");
 		PARSE_FLAG(NL80211_RRF_PTP_ONLY, "PTP-ONLY");
 		PARSE_FLAG(NL80211_RRF_AUTO_BW, "AUTO-BW");
-		PARSE_FLAG(NL80211_RRF_GO_CONCURRENT, "GO-CONCURRENT");
+		PARSE_FLAG(NL80211_RRF_IR_CONCURRENT, "IR-CONCURRENT");
 		PARSE_FLAG(NL80211_RRF_NO_HT40MINUS, "NO-HT40MINUS");
 		PARSE_FLAG(NL80211_RRF_NO_HT40PLUS, "NO-HT40PLUS");
 		PARSE_FLAG(NL80211_RRF_NO_80MHZ, "NO-80MHZ");
@@ -229,17 +228,15 @@ static int print_reg_handler(struct nl_msg *msg, void *arg)
 }
 
 static int handle_reg_dump(struct nl80211_state *state,
-			   struct nl_cb *cb,
 			   struct nl_msg *msg,
 			   int argc, char **argv,
 			   enum id_input id)
 {
-	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_reg_handler, NULL);
+	register_handler(print_reg_handler, NULL);
 	return 0;
 }
 
 static int handle_reg_get(struct nl80211_state *state,
-			  struct nl_cb *cb,
 			  struct nl_msg *msg,
 			  int argc, char **argv,
 			  enum id_input id)
@@ -250,7 +247,7 @@ static int handle_reg_get(struct nl80211_state *state,
 	err = handle_cmd(state, CIB_NONE, 2, dump_args);
 	/* dump might fail since it's not supported on older kernels */
 	if (err == -EOPNOTSUPP) {
-		nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_reg_handler,
+		register_handler(print_reg_handler,
 			  NULL);
 		return 0;
 	}
