@@ -338,12 +338,15 @@ static int print_sta_handler(struct nl_msg *msg, void *arg)
 		}
 	}
 
+	if (sinfo[NL80211_STA_INFO_CONNECTED_TIME])
+		printf("\n\tconnected time:\t%u seconds",
+			nla_get_u32(sinfo[NL80211_STA_INFO_CONNECTED_TIME]));
+
 	printf("\n");
 	return NL_SKIP;
 }
 
 static int handle_station_get(struct nl80211_state *state,
-			      struct nl_cb *cb,
 			      struct nl_msg *msg,
 			      int argc, char **argv,
 			      enum id_input id)
@@ -366,7 +369,7 @@ static int handle_station_get(struct nl80211_state *state,
 
 	NLA_PUT(msg, NL80211_ATTR_MAC, ETH_ALEN, mac_addr);
 
-	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_sta_handler, NULL);
+	register_handler(print_sta_handler, NULL);
 
 	return 0;
  nla_put_failure:
@@ -397,7 +400,6 @@ static const struct cmd *select_station_cmd(int argc, char **argv)
 }
 
 static int handle_station_set_plink(struct nl80211_state *state,
-			      struct nl_cb *cb,
 			      struct nl_msg *msg,
 			      int argc, char **argv,
 			      enum id_input id)
@@ -447,7 +449,6 @@ COMMAND_ALIAS(station, set, "<MAC address> plink_action <open|block>",
 	select_station_cmd, station_set_plink);
 
 static int handle_station_set_vlan(struct nl80211_state *state,
-				   struct nl_cb *cb,
 				   struct nl_msg *msg,
 				   int argc, char **argv,
 				   enum id_input id)
@@ -495,7 +496,6 @@ COMMAND_ALIAS(station, set, "<MAC address> vlan <ifindex>",
 	select_station_cmd, station_set_vlan);
 
 static int handle_station_set_mesh_power_mode(struct nl80211_state *state,
-					      struct nl_cb *cb,
 					      struct nl_msg *msg,
 					      int argc, char **argv,
 					      enum id_input id)
@@ -548,12 +548,11 @@ COMMAND_ALIAS(station, set, "<MAC address> mesh_power_mode "
 	select_station_cmd, station_set_mesh_power_mode);
 
 static int handle_station_dump(struct nl80211_state *state,
-			       struct nl_cb *cb,
 			       struct nl_msg *msg,
 			       int argc, char **argv,
 			       enum id_input id)
 {
-	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_sta_handler, NULL);
+	register_handler(print_sta_handler, NULL);
 	return 0;
 }
 COMMAND(station, dump, NULL,
